package com.loafle.overflow.module.member.service;

import com.loafle.overflow.module.email.service.EmailAuthService;
import com.loafle.overflow.module.member.dao.MemberDAO;
import com.loafle.overflow.module.member.exception.EmailNotConfirmedException;
import com.loafle.overflow.module.member.exception.JoinedEmailException;
import com.loafle.overflow.module.member.exception.SignInIdNotExistException;
import com.loafle.overflow.module.member.exception.SignInPwNotMatchException;
import com.loafle.overflow.module.member.model.Member;
import com.loafle.overflow.module.meta.model.MetaMemberStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

/**
 * Created by geek on 17. 6. 28.
 */
@Service("MemberService")
public class MemberService {

    @Autowired
    private MemberDAO memberDAO;

    @Autowired
    private EmailAuthService emailAuthService;

    public Member signin(String signinId, String signinPw) throws SignInIdNotExistException, EmailNotConfirmedException, SignInPwNotMatchException {
        Member m = this.memberDAO.findByEmail(signinId);

        if ( null == m ) {
            throw new SignInIdNotExistException();
        }

        if ( m.getStatus().getId() == 1 ) {
            throw new EmailNotConfirmedException();
        }

        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        Boolean match = passwordEncoder.matches(signinPw, m.getPw());
        if(!match) {
            throw new SignInPwNotMatchException();
        }

        return m;
    }

    public Member signup(Member member) {

        Member isMember = this.memberDAO.findByEmail(member.getEmail());

        if (null != isMember && isMember.getId() > 0) {
            throw new JoinedEmailException();
        }

        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        member.setPw(passwordEncoder.encode(member.getPw()));

        if (member.getStatus() == null) {
            member.setStatus(new MetaMemberStatus((short) 1));
        }

        Member resMember = this.memberDAO.save(member);

        this.emailAuthService.sendEmailByMember(member.getId(), member.getEmail());

        return resMember;
    }

    public void signout(Member member) {
        // Todo websocket session remove
    }

    public Member modify(Member member) {

        Member preMember = this.memberDAO.findByEmail(member.getEmail());

        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        Boolean match = passwordEncoder.matches(member.getPw(), preMember.getPw());
        if(!match) {
            throw new SignInPwNotMatchException();
        }

        return this.memberDAO.save(member);
    }

    public Member read(long memberId) {
        return this.memberDAO.findOne(memberId);
    }

    public void withdrawal(Member member) {
        // Todo DB delete?
    }

    public void emailConfirm(Member member) {

    }
}
